@@ -33,20 +33,11 @@
 
 from __future__ import annotations
 
-import copy
-import csv
 import io
-import json
-import math
-import random
 import re
-import textwrap
-from html import escape
 from html.parser import HTMLParser
 from typing import Any
 
-import wcwidth  # type: ignore
-
 # hrule styles
 FRAME = 0
 ALL = 1
@@ -322,13 +313,14 @@ class PrettyTable:
         elif isinstance(index, int):
             new.add_row(self._rows[index])
         else:
-            raise IndexError(f"Index {index} is invalid, must be an integer or slice")
+            msg = f"Index {index} is invalid, must be an integer or slice"
+            raise IndexError(msg)
         return new
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.get_string()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.get_string()
 
     def _repr_html_(self):
@@ -351,7 +343,7 @@ class PrettyTable:
     # Secondly, in the _get_options method, where keyword arguments are mixed with
     # persistent settings
 
-    def _validate_option(self, option, val):
+    def _validate_option(self, option, val) -> None:
         if option == "field_names":
             self._validate_field_names(val)
         elif option == "none_format":
@@ -422,64 +414,69 @@ class PrettyTable:
             try:
                 assert len(val) == len(self._field_names)
             except AssertionError:
-                raise ValueError(
+                msg = (
                     "Field name list has incorrect number of values, "
                     f"(actual) {len(val)}!={len(self._field_names)} (expected)"
                 )
+                raise ValueError(msg)
         if self._rows:
             try:
                 assert len(val) == len(self._rows[0])
             except AssertionError:
-                raise ValueError(
+                msg = (
                     "Field name list has incorrect number of values, "
                     f"(actual) {len(val)}!={len(self._rows[0])} (expected)"
                 )
+                raise ValueError(msg)
         # Check for uniqueness
         try:
             assert len(val) == len(set(val))
         except AssertionError:
-            raise ValueError("Field names must be unique")
+            msg = "Field names must be unique"
+            raise ValueError(msg)
 
     def _validate_none_format(self, val):
         try:
             if val is not None:
                 assert isinstance(val, str)
         except AssertionError:
-            raise TypeError(
-                "Replacement for None value must be a string if being supplied."
-            )
+            msg = "Replacement for None value must be a string if being supplied."
+            raise TypeError(msg)
 
     def _validate_header_style(self, val):
         try:
             assert val in ("cap", "title", "upper", "lower", None)
         except AssertionError:
-            raise ValueError(
-                "Invalid header style, use cap, title, upper, lower or None"
-            )
+            msg = "Invalid header style, use cap, title, upper, lower or None"
+            raise ValueError(msg)
 
     def _validate_align(self, val):
         try:
             assert val in ["l", "c", "r"]
         except AssertionError:
-            raise ValueError(f"Alignment {val} is invalid, use l, c or r")
+            msg = f"Alignment {val} is invalid, use l, c or r"
+            raise ValueError(msg)
 
     def _validate_valign(self, val):
         try:
             assert val in ["t", "m", "b", None]
         except AssertionError:
-            raise ValueError(f"Alignment {val} is invalid, use t, m, b or None")
+            msg = f"Alignment {val} is invalid, use t, m, b or None"
+            raise ValueError(msg)
 
     def _validate_nonnegative_int(self, name, val):
         try:
             assert int(val) >= 0
         except AssertionError:
-            raise ValueError(f"Invalid value for {name}: {val}")
+            msg = f"Invalid value for {name}: {val}"
+            raise ValueError(msg)
 
     def _validate_true_or_false(self, name, val):
         try:
             assert val in (True, False)
         except AssertionError:
-            raise ValueError(f"Invalid value for {name}. Must be True or False.")
+            msg = f"Invalid value for {name}. Must be True or False."
+            raise ValueError(msg)
 
     def _validate_int_format(self, name, val):
         if val == "":
@@ -488,9 +485,8 @@ class PrettyTable:
             assert isinstance(val, str)
             assert val.isdigit()
         except AssertionError:
-            raise ValueError(
-                f"Invalid value for {name}. Must be an integer format string."
-            )
+            msg = f"Invalid value for {name}. Must be an integer format string."
+            raise ValueError(msg)
 
     def _validate_float_format(self, name, val):
         if val == "":
@@ -507,54 +503,58 @@ class PrettyTable:
                 or (bits[1][-1] == "f" and bits[1].rstrip("f").isdigit())
             )
         except AssertionError:
-            raise ValueError(
-                f"Invalid value for {name}. Must be a float format string."
-            )
+            msg = f"Invalid value for {name}. Must be a float format string."
+            raise ValueError(msg)
 
     def _validate_function(self, name, val):
         try:
             assert hasattr(val, "__call__")
         except AssertionError:
-            raise ValueError(f"Invalid value for {name}. Must be a function.")
+            msg = f"Invalid value for {name}. Must be a function."
+            raise ValueError(msg)
 
     def _validate_hrules(self, name, val):
         try:
             assert val in (ALL, FRAME, HEADER, NONE)
         except AssertionError:
-            raise ValueError(
-                f"Invalid value for {name}. Must be ALL, FRAME, HEADER or NONE."
-            )
+            msg = f"Invalid value for {name}. Must be ALL, FRAME, HEADER or NONE."
+            raise ValueError(msg)
 
     def _validate_vrules(self, name, val):
         try:
             assert val in (ALL, FRAME, NONE)
         except AssertionError:
-            raise ValueError(f"Invalid value for {name}. Must be ALL, FRAME, or NONE.")
+            msg = f"Invalid value for {name}. Must be ALL, FRAME, or NONE."
+            raise ValueError(msg)
 
     def _validate_field_name(self, name, val):
         try:
             assert (val in self._field_names) or (val is None)
         except AssertionError:
-            raise ValueError(f"Invalid field name: {val}")
+            msg = f"Invalid field name: {val}"
+            raise ValueError(msg)
 
     def _validate_all_field_names(self, name, val):
         try:
             for x in val:
                 self._validate_field_name(name, x)
         except AssertionError:
-            raise ValueError("Fields must be a sequence of field names")
+            msg = "Fields must be a sequence of field names"
+            raise ValueError(msg)
 
     def _validate_single_char(self, name, val):
         try:
             assert _str_block_width(val) == 1
         except AssertionError:
-            raise ValueError(f"Invalid value for {name}. Must be a string of length 1.")
+            msg = f"Invalid value for {name}. Must be a string of length 1."
+            raise ValueError(msg)
 
     def _validate_attributes(self, name, val):
         try:
             assert isinstance(val, dict)
         except AssertionError:
-            raise TypeError("Attributes must be a dictionary of name/value pairs")
+            msg = "Attributes must be a dictionary of name/value pairs"
+            raise TypeError(msg)
 
     ##############################
     # ATTRIBUTE MANAGEMENT       #
@@ -573,7 +573,7 @@ class PrettyTable:
         return self._xhtml
 
     @xhtml.setter
-    def xhtml(self, val):
+    def xhtml(self, val) -> None:
         self._validate_option("xhtml", val)
         self._xhtml = val
 
@@ -603,7 +603,7 @@ class PrettyTable:
         return self._field_names
 
     @field_names.setter
-    def field_names(self, val):
+    def field_names(self, val) -> None:
         val = [str(x) for x in val]
         self._validate_option("field_names", val)
         old_names = None
@@ -639,7 +639,7 @@ class PrettyTable:
         return self._align
 
     @align.setter
-    def align(self, val):
+    def align(self, val) -> None:
         if val is None or (isinstance(val, dict) and len(val) == 0):
             if not self._field_names:
                 self._align = {BASE_ALIGN_VALUE: "c"}
@@ -663,7 +663,7 @@ class PrettyTable:
         return self._valign
 
     @valign.setter
-    def valign(self, val):
+    def valign(self, val) -> None:
         if not self._field_names:
             self._valign = {}
         elif val is None or (isinstance(val, dict) and len(val) == 0):
@@ -683,7 +683,7 @@ class PrettyTable:
         return self._max_width
 
     @max_width.setter
-    def max_width(self, val):
+    def max_width(self, val) -> None:
         if val is None or (isinstance(val, dict) and len(val) == 0):
             self._max_width = {}
         else:
@@ -700,7 +700,7 @@ class PrettyTable:
         return self._min_width
 
     @min_width.setter
-    def min_width(self, val):
+    def min_width(self, val) -> None:
         if val is None or (isinstance(val, dict) and len(val) == 0):
             self._min_width = {}
         else:
@@ -713,7 +713,7 @@ class PrettyTable:
         return self._min_table_width
 
     @min_table_width.setter
-    def min_table_width(self, val):
+    def min_table_width(self, val) -> None:
         self._validate_option("min_table_width", val)
         self._min_table_width = val
 
@@ -722,7 +722,7 @@ class PrettyTable:
         return self._max_table_width
 
     @max_table_width.setter
-    def max_table_width(self, val):
+    def max_table_width(self, val) -> None:
         self._validate_option("max_table_width", val)
         self._max_table_width = val
 
@@ -732,7 +732,7 @@ class PrettyTable:
         return self._fields
 
     @fields.setter
-    def fields(self, val):
+    def fields(self, val) -> None:
         self._validate_option("fields", val)
         self._fields = val
 
@@ -746,7 +746,7 @@ class PrettyTable:
         return self._title
 
     @title.setter
-    def title(self, val):
+    def title(self, val) -> None:
         self._title = str(val)
 
     @property
@@ -759,7 +759,7 @@ class PrettyTable:
         return self._start
 
     @start.setter
-    def start(self, val):
+    def start(self, val) -> None:
         self._validate_option("start", val)
         self._start = val
 
@@ -773,7 +773,7 @@ class PrettyTable:
         return self._end
 
     @end.setter
-    def end(self, val):
+    def end(self, val) -> None:
         self._validate_option("end", val)
         self._end = val
 
@@ -787,7 +787,7 @@ class PrettyTable:
         return self._sortby
 
     @sortby.setter
-    def sortby(self, val):
+    def sortby(self, val) -> None:
         self._validate_option("sortby", val)
         self._sortby = val
 
@@ -802,7 +802,7 @@ class PrettyTable:
         return self._reversesort
 
     @reversesort.setter
-    def reversesort(self, val):
+    def reversesort(self, val) -> None:
         self._validate_option("reversesort", val)
         self._reversesort = val
 
@@ -817,7 +817,7 @@ class PrettyTable:
         return self._sort_key
 
     @sort_key.setter
-    def sort_key(self, val):
+    def sort_key(self, val) -> None:
         self._validate_option("sort_key", val)
         self._sort_key = val
 
@@ -831,7 +831,7 @@ class PrettyTable:
         return self._header
 
     @header.setter
-    def header(self, val):
+    def header(self, val) -> None:
         self._validate_option("header", val)
         self._header = val
 
@@ -846,7 +846,7 @@ class PrettyTable:
         return self._header_style
 
     @header_style.setter
-    def header_style(self, val):
+    def header_style(self, val) -> None:
         self._validate_header_style(val)
         self._header_style = val
 
@@ -860,7 +860,7 @@ class PrettyTable:
         return self._border
 
     @border.setter
-    def border(self, val):
+    def border(self, val) -> None:
         self._validate_option("border", val)
         self._border = val
 
@@ -875,7 +875,7 @@ class PrettyTable:
         return self._preserve_internal_border
 
     @preserve_internal_border.setter
-    def preserve_internal_border(self, val):
+    def preserve_internal_border(self, val) -> None:
         self._validate_option("preserve_internal_border", val)
         self._preserve_internal_border = val
 
@@ -889,7 +889,7 @@ class PrettyTable:
         return self._hrules
 
     @hrules.setter
-    def hrules(self, val):
+    def hrules(self, val) -> None:
         self._validate_option("hrules", val)
         self._hrules = val
 
@@ -903,7 +903,7 @@ class PrettyTable:
         return self._vrules
 
     @vrules.setter
-    def vrules(self, val):
+    def vrules(self, val) -> None:
         self._validate_option("vrules", val)
         self._vrules = val
 
@@ -916,7 +916,7 @@ class PrettyTable:
         return self._int_format
 
     @int_format.setter
-    def int_format(self, val):
+    def int_format(self, val) -> None:
         if val is None or (isinstance(val, dict) and len(val) == 0):
             self._int_format = {}
         else:
@@ -933,7 +933,7 @@ class PrettyTable:
         return self._float_format
 
     @float_format.setter
-    def float_format(self, val):
+    def float_format(self, val) -> None:
         if val is None or (isinstance(val, dict) and len(val) == 0):
             self._float_format = {}
         else:
@@ -962,9 +962,8 @@ class PrettyTable:
             for field in self._field_names:
                 self._custom_format[field] = val
         else:
-            raise TypeError(
-                "The custom_format property need to be a dictionary or callable"
-            )
+            msg = "The custom_format property need to be a dictionary or callable"
+            raise TypeError(msg)
 
     @property
     def padding_width(self):
@@ -976,7 +975,7 @@ class PrettyTable:
         return self._padding_width
 
     @padding_width.setter
-    def padding_width(self, val):
+    def padding_width(self, val) -> None:
         self._validate_option("padding_width", val)
         self._padding_width = val
 
@@ -990,7 +989,7 @@ class PrettyTable:
         return self._left_padding_width
 
     @left_padding_width.setter
-    def left_padding_width(self, val):
+    def left_padding_width(self, val) -> None:
         self._validate_option("left_padding_width", val)
         self._left_padding_width = val
 
@@ -1004,7 +1003,7 @@ class PrettyTable:
         return self._right_padding_width
 
     @right_padding_width.setter
-    def right_padding_width(self, val):
+    def right_padding_width(self, val) -> None:
         self._validate_option("right_padding_width", val)
         self._right_padding_width = val
 
@@ -1018,7 +1017,7 @@ class PrettyTable:
         return self._vertical_char
 
     @vertical_char.setter
-    def vertical_char(self, val):
+    def vertical_char(self, val) -> None:
         val = str(val)
         self._validate_option("vertical_char", val)
         self._vertical_char = val
@@ -1033,7 +1032,7 @@ class PrettyTable:
         return self._horizontal_char
 
     @horizontal_char.setter
-    def horizontal_char(self, val):
+    def horizontal_char(self, val) -> None:
         val = str(val)
         self._validate_option("horizontal_char", val)
         self._horizontal_char = val
@@ -1048,7 +1047,7 @@ class PrettyTable:
         return self._bottom_left_junction_char or self.junction_char
 
     @horizontal_align_char.setter
-    def horizontal_align_char(self, val):
+    def horizontal_align_char(self, val) -> None:
         val = str(val)
         self._validate_option("horizontal_align_char", val)
         self._horizontal_align_char = val
@@ -1063,7 +1062,7 @@ class PrettyTable:
         return self._junction_char
 
     @junction_char.setter
-    def junction_char(self, val):
+    def junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("junction_char", val)
         self._junction_char = val
@@ -1078,7 +1077,7 @@ class PrettyTable:
         return self._top_junction_char or self.junction_char
 
     @top_junction_char.setter
-    def top_junction_char(self, val):
+    def top_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("top_junction_char", val)
         self._top_junction_char = val
@@ -1094,7 +1093,7 @@ class PrettyTable:
         return self._bottom_junction_char or self.junction_char
 
     @bottom_junction_char.setter
-    def bottom_junction_char(self, val):
+    def bottom_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("bottom_junction_char", val)
         self._bottom_junction_char = val
@@ -1110,7 +1109,7 @@ class PrettyTable:
         return self._right_junction_char or self.junction_char
 
     @right_junction_char.setter
-    def right_junction_char(self, val):
+    def right_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("right_junction_char", val)
         self._right_junction_char = val
@@ -1125,7 +1124,7 @@ class PrettyTable:
         return self._left_junction_char or self.junction_char
 
     @left_junction_char.setter
-    def left_junction_char(self, val):
+    def left_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("left_junction_char", val)
         self._left_junction_char = val
@@ -1142,7 +1141,7 @@ class PrettyTable:
         return self._top_right_junction_char or self.junction_char
 
     @top_right_junction_char.setter
-    def top_right_junction_char(self, val):
+    def top_right_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("top_right_junction_char", val)
         self._top_right_junction_char = val
@@ -1159,7 +1158,7 @@ class PrettyTable:
         return self._top_left_junction_char or self.junction_char
 
     @top_left_junction_char.setter
-    def top_left_junction_char(self, val):
+    def top_left_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("top_left_junction_char", val)
         self._top_left_junction_char = val
@@ -1176,7 +1175,7 @@ class PrettyTable:
         return self._bottom_right_junction_char or self.junction_char
 
     @bottom_right_junction_char.setter
-    def bottom_right_junction_char(self, val):
+    def bottom_right_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("bottom_right_junction_char", val)
         self._bottom_right_junction_char = val
@@ -1193,7 +1192,7 @@ class PrettyTable:
         return self._bottom_left_junction_char or self.junction_char
 
     @bottom_left_junction_char.setter
-    def bottom_left_junction_char(self, val):
+    def bottom_left_junction_char(self, val) -> None:
         val = str(val)
         self._validate_option("bottom_left_junction_char", val)
         self._bottom_left_junction_char = val
@@ -1208,7 +1207,7 @@ class PrettyTable:
         return self._format
 
     @format.setter
-    def format(self, val):
+    def format(self, val) -> None:
         self._validate_option("format", val)
         self._format = val
 
@@ -1223,7 +1222,7 @@ class PrettyTable:
         return self._print_empty
 
     @print_empty.setter
-    def print_empty(self, val):
+    def print_empty(self, val) -> None:
         self._validate_option("print_empty", val)
         self._print_empty = val
 
@@ -1238,7 +1237,7 @@ class PrettyTable:
         return self._attributes
 
     @attributes.setter
-    def attributes(self, val):
+    def attributes(self, val) -> None:
         self._validate_option("attributes", val)
         self._attributes = val
 
@@ -1248,7 +1247,7 @@ class PrettyTable:
         return self._oldsortslice
 
     @oldsortslice.setter
-    def oldsortslice(self, val):
+    def oldsortslice(self, val) -> None:
         self._validate_option("oldsortslice", val)
         self._oldsortslice = val
 
@@ -1288,13 +1287,14 @@ class PrettyTable:
         elif style == RANDOM:
             self._set_random_style()
         else:
-            raise ValueError("Invalid pre-set style")
+            msg = "Invalid pre-set style"
+            raise ValueError(msg)
 
-    def _set_orgmode_style(self):
+    def _set_orgmode_style(self) -> None:
         self._set_default_style()
         self.orgmode = True
 
-    def _set_markdown_style(self):
+    def _set_markdown_style(self) -> None:
         self.header = True
         self.border = True
         self._hrules = None
@@ -1305,7 +1305,7 @@ class PrettyTable:
         self.junction_char = "|"
         self._horizontal_align_char = ":"
 
-    def _set_default_style(self):
+    def _set_default_style(self) -> None:
         self.header = True
         self.border = True
         self._hrules = FRAME
@@ -1326,7 +1326,7 @@ class PrettyTable:
         self._bottom_right_junction_char = None
         self._bottom_left_junction_char = None
 
-    def _set_msword_style(self):
+    def _set_msword_style(self) -> None:
         self.header = True
         self.border = True
         self._hrules = NONE
@@ -1335,14 +1335,14 @@ class PrettyTable:
         self.right_padding_width = 1
         self.vertical_char = "|"
 
-    def _set_columns_style(self):
+    def _set_columns_style(self) -> None:
         self.header = True
         self.border = False
         self.padding_width = 1
         self.left_padding_width = 0
         self.right_padding_width = 8
 
-    def _set_double_border_style(self):
+    def _set_double_border_style(self) -> None:
         self.horizontal_char = "═"
         self.vertical_char = "║"
         self.junction_char = "╬"
@@ -1355,7 +1355,7 @@ class PrettyTable:
         self.bottom_right_junction_char = "╝"
         self.bottom_left_junction_char = "╚"
 
-    def _set_single_border_style(self):
+    def _set_single_border_style(self) -> None:
         self.horizontal_char = "─"
         self.vertical_char = "│"
         self.junction_char = "┼"
@@ -1368,8 +1368,10 @@ class PrettyTable:
         self.bottom_right_junction_char = "┘"
         self.bottom_left_junction_char = "└"
 
-    def _set_random_style(self):
+    def _set_random_style(self) -> None:
         # Just for fun!
+        import random
+
         self.header = random.choice((True, False))
         self.border = random.choice((True, False))
         self._hrules = random.choice((ALL, FRAME, HEADER, NONE))
@@ -1395,7 +1397,7 @@ class PrettyTable:
         for row in rows:
             self.add_row(row)
 
-    def add_row(self, row, *, divider=False) -> None:
+    def add_row(self, row, *, divider: bool = False) -> None:
         """Add a row to the table
 
         Arguments:
@@ -1404,10 +1406,11 @@ class PrettyTable:
         has fields"""
 
         if self._field_names and len(row) != len(self._field_names):
-            raise ValueError(
+            msg = (
                 "Row has incorrect number of values, "
                 f"(actual) {len(row)}!={len(self._field_names)} (expected)"
             )
+            raise ValueError(msg)
         if not self._field_names:
             self.field_names = [f"Field {n + 1}" for n in range(0, len(row))]
         self._rows.append(list(row))
@@ -1421,10 +1424,11 @@ class PrettyTable:
         row_index - The index of the row you want to delete.  Indexing starts at 0."""
 
         if row_index > len(self._rows) - 1:
-            raise IndexError(
+            msg = (
                 f"Can't delete row at index {row_index}, "
                 f"table only has {len(self._rows)} rows"
             )
+            raise IndexError(msg)
         del self._rows[row_index]
         del self._dividers[row_index]
 
@@ -1455,12 +1459,13 @@ class PrettyTable:
                     self._dividers.append(False)
                 self._rows[i].append(column[i])
         else:
-            raise ValueError(
+            msg = (
                 f"Column length {len(column)} does not match number of rows "
                 f"{len(self._rows)}"
             )
+            raise ValueError(msg)
 
-    def add_autoindex(self, fieldname: str = "Index"):
+    def add_autoindex(self, fieldname: str = "Index") -> None:
         """Add an auto-incrementing index column to the table.
         Arguments:
         fieldname - name of the field to contain the new column of data"""
@@ -1478,11 +1483,13 @@ class PrettyTable:
         fieldname - The field name of the column you want to delete."""
 
         if fieldname not in self._field_names:
-            raise ValueError(
-                "Can't delete column %r which is not a field name of this table."
-                " Field names are: %s"
-                % (fieldname, ", ".join(map(repr, self._field_names)))
+            msg = (
+                "Can't delete column {!r} which is not a field name of this table."
+                " Field names are: {}".format(
+                    fieldname, ", ".join(map(repr, self._field_names))
+                )
             )
+            raise ValueError(msg)
 
         col_index = self._field_names.index(fieldname)
         del self._field_names[col_index]
@@ -1509,8 +1516,34 @@ class PrettyTable:
     ##############################
 
     def copy(self):
+        import copy
+
         return copy.deepcopy(self)
 
+    def get_formatted_string(self, out_format: str = "text", **kwargs) -> str:
+        """Return string representation of specified format of table in current state.
+
+        Arguments:
+        out_format - resulting table format
+        kwargs - passed through to function that performs formatting
+        """
+        if out_format == "text":
+            return self.get_string(**kwargs)
+        if out_format == "html":
+            return self.get_html_string(**kwargs)
+        if out_format == "json":
+            return self.get_json_string(**kwargs)
+        if out_format == "csv":
+            return self.get_csv_string(**kwargs)
+        if out_format == "latex":
+            return self.get_latex_string(**kwargs)
+
+        msg = (
+            f"Invalid format {out_format}. "
+            "Must be one of: text, html, json, csv, or latex"
+        )
+        raise ValueError(msg)
+
     ##############################
     # MISC PRIVATE METHODS       #
     ##############################
@@ -1534,7 +1567,7 @@ class PrettyTable:
                 table_width += self._widths[index] + per_col_padding
         return table_width
 
-    def _compute_widths(self, rows, options):
+    def _compute_widths(self, rows, options) -> None:
         if options["header"]:
             widths = [_get_size(field)[0] for field in self._field_names]
         else:
@@ -1563,7 +1596,7 @@ class PrettyTable:
             if table_width > self._max_table_width:
                 # Shrink widths in proportion
                 scale = 1.0 * self._max_table_width / table_width
-                widths = [int(math.floor(w * scale)) for w in widths]
+                widths = [int(w * scale) for w in widths]
                 self._widths = widths
 
         # Are we under min_table_width or title width?
@@ -1595,7 +1628,7 @@ class PrettyTable:
             if content_width < min_width:
                 # Grow widths in proportion
                 scale = 1.0 * min_width / content_width
-                widths = [int(math.floor(w * scale)) for w in widths]
+                widths = [int(w * scale) for w in widths]
                 if sum(widths) < min_width:
                     widths[-1] += min_width - sum(widths)
                 self._widths = widths
@@ -1618,6 +1651,7 @@ class PrettyTable:
         Arguments:
 
         options - dictionary of option settings."""
+        import copy
 
         if options["oldsortslice"]:
             rows = copy.deepcopy(self._rows[options["start"] : options["end"]])
@@ -1646,6 +1680,7 @@ class PrettyTable:
         Arguments:
 
         options - dictionary of option settings."""
+        import copy
 
         if options["oldsortslice"]:
             dividers = copy.deepcopy(self._dividers[options["start"] : options["end"]])
@@ -1771,15 +1806,16 @@ class PrettyTable:
         if options["border"] and options["hrules"] == FRAME:
             lines.append(self._stringify_hrule(options, where="bottom_"))
 
-        if "orgmode" in self.__dict__ and self.orgmode is True:
-            tmp = list()
-            for line in lines:
-                tmp.extend(line.split("\n"))
-            lines = ["|" + line[1:-1] + "|" for line in tmp]
+        if "orgmode" in self.__dict__ and self.orgmode:
+            lines = [
+                "|" + new_line[1:-1] + "|"
+                for old_line in lines
+                for new_line in old_line.split("\n")
+            ]
 
         return "\n".join(lines)
 
-    def _stringify_hrule(self, options, where=""):
+    def _stringify_hrule(self, options, where: str = ""):
         if not options["border"] and not options["preserve_internal_border"]:
             return ""
         lpad, rpad = self._get_padding_widths(options)
@@ -1800,9 +1836,9 @@ class PrettyTable:
             # If necessary, add column alignment characters (e.g. ":" for Markdown)
             if self._horizontal_align_char:
                 if self._align[field] in ("l", "c"):
-                    line = self._horizontal_align_char + line[1:]
+                    line = " " + self._horizontal_align_char + line[2:]
                 if self._align[field] in ("c", "r"):
-                    line = line[:-1] + self._horizontal_align_char
+                    line = line[:-2] + self._horizontal_align_char + " "
 
             bits.append(line)
             if options["vrules"] == ALL:
@@ -1908,6 +1944,8 @@ class PrettyTable:
         return "".join(bits)
 
     def _stringify_row(self, row, options, hrule):
+        import textwrap
+
         for index, field, value, width in zip(
             range(0, len(row)), self._field_names, row, self._widths
         ):
@@ -2019,6 +2057,7 @@ class PrettyTable:
         header as a PrettyTable formatting option (skip the header row) and
         delimiter as a csv.writer keyword argument.
         """
+        import csv
 
         options = self._get_options(kwargs)
         csv_options = {
@@ -2046,9 +2085,10 @@ class PrettyTable:
         a PrettyTable formatting option (skip the header row) and indent as a
         json.dumps keyword argument.
         """
+        import json
 
         options = self._get_options(kwargs)
-        json_options: Any = dict(indent=4, separators=(",", ": "), sort_keys=True)
+        json_options: Any = {"indent": 4, "separators": (",", ": "), "sort_keys": True}
         json_options.update(
             {key: value for key, value in kwargs.items() if key not in options}
         )
@@ -2108,6 +2148,8 @@ class PrettyTable:
         return string
 
     def _get_simple_html_string(self, options):
+        from html import escape
+
         lines = []
         if options["xhtml"]:
             linebreak = "<br/>"
@@ -2117,14 +2159,16 @@ class PrettyTable:
         open_tag = ["<table"]
         if options["attributes"]:
             for attr_name in options["attributes"]:
-                open_tag.append(f' {attr_name}="{options["attributes"][attr_name]}"')
+                open_tag.append(
+                    f' {escape(attr_name)}="{escape(options["attributes"][attr_name])}"'
+                )
         open_tag.append(">")
         lines.append("".join(open_tag))
 
         # Title
         title = options["title"] or self._title
         if title:
-            lines.append(f"    <caption>{title}</caption>")
+            lines.append(f"    <caption>{escape(title)}</caption>")
 
         # Headers
         if options["header"]:
@@ -2158,6 +2202,8 @@ class PrettyTable:
         return "\n".join(lines)
 
     def _get_formatted_html_string(self, options):
+        from html import escape
+
         lines = []
         lpad, rpad = self._get_padding_widths(options)
         if options["xhtml"]:
@@ -2185,14 +2231,16 @@ class PrettyTable:
             open_tag.append(' rules="cols"')
         if options["attributes"]:
             for attr_name in options["attributes"]:
-                open_tag.append(f' {attr_name}="{options["attributes"][attr_name]}"')
+                open_tag.append(
+                    f' {escape(attr_name)}="{escape(options["attributes"][attr_name])}"'
+                )
         open_tag.append(">")
         lines.append("".join(open_tag))
 
         # Title
         title = options["title"] or self._title
         if title:
-            lines.append(f"    <caption>{title}</caption>")
+            lines.append(f"    <caption>{escape(title)}</caption>")
 
         # Headers
         if options["header"]:
@@ -2374,6 +2422,8 @@ class PrettyTable:
 
 
 def _str_block_width(val):
+    import wcwidth  # type: ignore[import-not-found]
+
     return wcwidth.wcswidth(_re.sub("", val))
 
 
@@ -2383,6 +2433,8 @@ def _str_block_width(val):
 
 
 def from_csv(fp, field_names: Any | None = None, **kwargs):
+    import csv
+
     fmtparams = {}
     for param in [
         "delimiter",
@@ -2425,6 +2477,8 @@ def from_db_cursor(cursor, **kwargs):
 
 
 def from_json(json_string, **kwargs):
+    import json
+
     table = PrettyTable(**kwargs)
     objects = json.loads(json_string)
     table.field_names = objects[0]
@@ -2528,7 +2582,6 @@ def from_html_one(html_code, **kwargs):
     try:
         assert len(tables) == 1
     except AssertionError:
-        raise ValueError(
-            "More than one <table> in provided HTML code. Use from_html instead."
-        )
+        msg = "More than one <table> in provided HTML code. Use from_html instead."
+        raise ValueError(msg)
     return tables[0]
